"use strict"

/** myPrayerJournal script */
window.mpj = {
  /**
   * Show a message via toast
   * @param {string} message The message to show
   */
  showToast (message) {
    const [level, msg] = message.split("|||")
    
    let header
    if (level !== "success") {
      const heading = typ => `<span class="me-auto"><strong>${typ.toUpperCase()}</strong></span>`
      
      header = document.createElement("div")
      header.className = "toast-header"
      header.innerHTML = heading(level === "warning" ? level : "error")
      
      const close = document.createElement("button")
      close.type = "button"
      close.className = "btn-close"
      close.setAttribute("data-bs-dismiss", "toast")
      close.setAttribute("aria-label", "Close")
      header.appendChild(close)
    }

    const body = document.createElement("div")
    body.className = "toast-body"
    body.innerText = msg
    
    const toastEl = document.createElement("div")
    toastEl.className = `toast bg-${level === "error" ? "danger" : level} text-white`
    toastEl.setAttribute("role", "alert")
    toastEl.setAttribute("aria-live", "assertlive")
    toastEl.setAttribute("aria-atomic", "true")
    toastEl.addEventListener("hidden.bs.toast", e => e.target.remove())
    if (header) toastEl.appendChild(header)
    
    toastEl.appendChild(body)
    document.getElementById("toasts").appendChild(toastEl)
    new bootstrap.Toast(toastEl, { autohide: level === "success" }).show()
  },
  /**
   * Load local version of Bootstrap CSS if the CDN load failed
   */
  ensureCss () {
    let loaded = false
    for (let i = 0; !loaded && i < document.styleSheets.length; i++) {
      loaded = document.styleSheets[i].href.endsWith("bootstrap.min.css")
    }
    if (!loaded) {
      const css = document.createElement("link")
      css.rel = "stylesheet"
      css.href = "/style/bootstrap.min.css"
      document.getElementsByTagName("head")[0].appendChild(css)
    }
  },
  /** Script for the request edit component */
  edit: {
    /**
     * Toggle the recurrence input fields
     * @param {Event} e The click event
     */
    toggleRecurrence ({ target }) {
      const isDisabled = target.value === "Immediate"
      ;["recurCount", "recurInterval"].forEach(it => document.getElementById(it).disabled = isDisabled)
    }
  },
  /**
   * The time zone of the current browser
   * @type {string}
   **/
  timeZone: undefined,
  /**
   * Derive the time zone from the current browser
   */
  deriveTimeZone () {
    try {
      this.timeZone = (new Intl.DateTimeFormat()).resolvedOptions().timeZone
    } catch (_) { }
  }
}

htmx.on("htmx:afterOnLoad", function (evt) {
  const hdrs = evt.detail.xhr.getAllResponseHeaders()
  // Show a message if there was one in the response
  if (hdrs.indexOf("x-toast") >= 0) {
    mpj.showToast(evt.detail.xhr.getResponseHeader("x-toast"))
  }
  // Hide a modal window if requested
  if (hdrs.indexOf("x-hide-modal") >= 0) {
    document.getElementById(evt.detail.xhr.getResponseHeader("x-hide-modal") + "Dismiss").click()
  }
})

htmx.on("htmx:configRequest", function (evt) {
  // Send the user's current time zone so that we can display local time
  if (mpj.timeZone) {
    evt.detail.headers["X-Time-Zone"] = mpj.timeZone
  }
})

mpj.deriveTimeZone()

class RelativeDateTime extends HTMLElement {
  static observedAttributes = ["interval"]

  static #LessThanXMinutes = Symbol()
  static #XMinutes = Symbol()
  static #AboutXHours = Symbol()
  static #XDays = Symbol()
  static #AboutXMonths = Symbol()
  static #XMonths = Symbol()
  static #AboutXYears = Symbol()
  static #OverXYears = Symbol()
  static #AlmostXYears = Symbol()

  static #messages = new Map([
    [RelativeDateTime.#LessThanXMinutes, ['less than a minute', 'less than %d minutes']],
    [RelativeDateTime.#XMinutes, ['a minute', '%d minutes']],
    [RelativeDateTime.#AboutXHours, ['about an hour', 'about %d hours']],
    [RelativeDateTime.#XDays, ['a day', '%d days']],
    [RelativeDateTime.#AboutXMonths, ['about a month', 'about %d months']],
    [RelativeDateTime.#XMonths, ['a month', '%d months']],
    [RelativeDateTime.#AboutXYears, ['about a year', 'about %d years']],
    [RelativeDateTime.#OverXYears, ['over a year', 'over %d years']],
    [RelativeDateTime.#AlmostXYears, ['almost a year', 'almost %d years']],
  ])

  static #aDay = 1440.0
  static #almost2Days = 2520.0
  static #aMonth = 43200.0
  static #twoMonths = 86400.0

  /** @type Date The date/time for this relative date */
  #jsDate
  /** @type ?number The interval timer for this element */
  #timeOut = null

  constructor() {
    super();
  }

  #update() {

    const now = new Date()
    const minutes = Math.abs((this.#jsDate.getTime() - now.getTime()) / 60 / 1000);
    const months = Math.round(minutes / RelativeDateTime.#aMonth);
    const years = Math.floor(months / 12);

    /** @type symbol */
    let typ
    /** @type number */
    let nbr

    if (minutes < 1.0) {
      typ = RelativeDateTime.#LessThanXMinutes
      nbr = 1
    } else if (minutes < 45.0) {
      typ = RelativeDateTime.#XMinutes
      nbr = Math.round(minutes)
    } else if (minutes < 90.0) {
      typ = RelativeDateTime.#AboutXHours
      nbr = 1
    } else if (minutes < RelativeDateTime.#aDay) {
      typ = RelativeDateTime.#AboutXHours
      nbr = Math.round(minutes / 60)
    } else if (minutes < RelativeDateTime.#almost2Days) {
      typ = RelativeDateTime.#XDays
      nbr = 1
    } else if (minutes < RelativeDateTime.#aMonth) {
      typ = RelativeDateTime.#XDays
      nbr = Math.round(minutes / RelativeDateTime.#aDay)
    } else if (minutes < RelativeDateTime.#twoMonths) {
      typ = RelativeDateTime.#AboutXMonths
      nbr = Math.round(minutes / RelativeDateTime.#aMonth)
    } else if (months < 12) {
      typ = RelativeDateTime.#XMonths
      nbr = Math.round(minutes / RelativeDateTime.#aMonth)
    } else if (months % 12 < 3) {
      typ = RelativeDateTime.#AboutXYears
      nbr = years
    } else if (months % 12 < 9) {
      typ = RelativeDateTime.#OverXYears
      nbr = years
    } else {
      typ = RelativeDateTime.#AlmostXYears
      nbr = years + 1
    }

    const tmpl = RelativeDateTime.#messages.get(typ)
    const message = nbr === 1 ? tmpl[0] : tmpl[1].replace("%d", nbr.toString())
    this.innerText = this.#jsDate < now ? `${message} ago` : `in ${message}`
  }

  connectedCallback() {
    this.#jsDate = new Date(this.innerText)
    this.#update()
  }

  disconnectedCallback() {
    if (this.#timeOut) clearInterval(this.#timeOut)
  }

  attributeChangedCallback(name, oldValue, newValue) {
    if (this.#timeOut) clearInterval(this.#timeOut)
    if (this.#jsDate) this.#update()
    this.#timeOut = setInterval(() => this.#update, parseInt(newValue))
  }
}

customElements.define("relative-date-time", RelativeDateTime)
